#!/usr/bin/ruby
#
# ACPI On Screen Displayer
#
# By: YOKOTA Hiroshi
#
# Licenced by GPL.
#
# This program required "libxosd".
#

require "thread"
require "socket"

class XOsd
  def initialize(opt = "")
    @fd = open("|osd_cat #{opt}", "w")
  end

  def print(str)
    @fd.print(str)
  end

  def close()
    @fd.close
  end
end

func = ""

osd = XOsd.new("-O 3 -u white -f -adobe-courier-bold-r-normal-*-20-*-*-*-*-*-*-* -c green -l 1 -d 3")


def get_mute()
  ret = nil
  open("/proc/acpi/pcc/mute", "r") {
    |fd|
    ret = if fd.gets.to_i == 0
      "off"
    else
      "on"
    end
  }
  return ret
end

def get_vol()
  left = right = nil
  open("|amixer sget PCM", "r") {
    |fd|
    while l = fd.gets()
      case l
      when / Front Left:/
        l =~ /\[([0-9]+)%\]/
        left = $1.to_i
      when / Front Right:/
        l =~ /\[([0-9]+)%\]/
        right = $1.to_i
      end
    end
  }
  return {"left" => left, "right" => right}
end

def get_vga()
  ac_state = bright = nil

  open("/proc/acpi/ac_adapter/AC/state", "r") {
    |fd|
    ac_state = fd.gets.split[1]
  }
  batt_mode = if ac_state == "on-line" then "ac" else "dc" end

  open("/proc/acpi/pcc/#{batt_mode}_brightness", "r") {
    |fd|
    bright = fd.gets.to_i
  }

  return {"batt_mode" => batt_mode, "bright" => bright}
end


UNIXSocket.open("/var/run/acpid.socket") {
    |fd|
    while l = fd.gets
      if l.split[0] != "pcc"
        next
      end

      key = l.split[3].hex
      case key
      when 1,2,0x81,0x82
        tmp = get_vga()
        func = "backlight #{tmp["batt_mode"]} #{tmp["bright"]}"
      when 3
        func = "grafics"

      when 4,0x84
        func = "mute " + get_mute()

      when 5,6,0x85,0x86
        tmp =  get_vol()
        func = "vol L:#{tmp["left"]} R:#{tmp["right"]}"
      else
        func = "???"
      end

      osd.print("#{func}\n")

    end
}


osd.close

# end
